@@ -12,6 +12,7 @@ from custom_inherit import doc_inherit
 
 from pi_touch_gui._utilities import snapped_value_string
 from pi_touch_gui._widget_bases import IWidget, FONT_NAME
+from pi_touch_gui.widget_functions import LightFunction, IndicatorFunction, GraphFunction
 
 LOG = logging.getLogger(__name__)
 
@@ -58,18 +59,72 @@ class Label(IWidget):
                                     label_color1=label_color1,
                                     label_color2=label_color1)
 
-    def render(self, surface):
-        color, label_color = self.state_colors()
+    def render(self, page, surface, label=None, active=None):
+        label = label or self.label
+        color, label_color = self.state_colors(active)
 
         if self.alignment == Label.LEFT:
-            self.render_left_text(surface, self.label, label_color)
+            self.render_left_text(surface, label, label_color)
         elif self.alignment == Label.CENTER:
-            self.render_centered_text(surface, self.label, label_color)
+            self.render_centered_text(surface, label, label_color)
         elif self.alignment == Label.RIGHT:
-            self.render_right_text(surface, self.label, label_color)
+            self.render_right_text(surface, label, label_color)
 
-    def can_focus(self):
-        return False
+
+class Indicator(Label):
+    """ Indicators are dynamic Labels, controlled by their function.
+    """
+
+    @doc_inherit(IWidget.__init__, style='numpy_with_merge')
+    def __init__(self, position, size,
+                 function=None,
+                 **kwargs):
+        """ Initialize the Indicator.
+
+        If color1 is None, the indicator is colored like a label, with only
+        the label colors being used.
+
+        If color1 is specified, then the indicator is set in a filled
+        rectangle with the active/inactive color.
+
+        Parameters
+        ----------
+        function : IndicatorFunction
+            The function that defines the content and state of the indicator.
+        """
+        # Set up the boxing behavior and font size.
+        if (function is not None
+                and not issubclass(function.__class__, IndicatorFunction)):
+            raise ValueError(f"Function type {type(function)} is not a "
+                             f"IndicatorFunction.")
+        self.function = function
+
+        kw_font_size = kwargs.pop('font_size', None)
+        if kwargs.get('color1') is None:
+            self._box = True
+            font_size = kw_font_size or (size[1] >> 1)
+        else:
+            self._box = False
+            font_size = kw_font_size or size[1]
+
+        super(Indicator, self).__init__(position, size,
+                                        font_size=font_size,
+                                        **kwargs)
+
+    def render(self, page, surface, label=None, active=None):
+        if self.function:
+            active, label = self.function.status(page, self, label, active)
+        else:
+            active, label = False, None
+
+        color, _ = self.state_colors(active=active)
+
+        if self._box:
+            pygame.draw.rect(surface, color, (self.position, self.size), 0)
+
+        super(Indicator, self).render(page, surface,
+                                      label=label,
+                                      active=active)
 
 
 class Light(IWidget):
@@ -88,87 +143,32 @@ class Light(IWidget):
         size : Tuple[int, int]
             The (w, h) of the Light's rectangular extent.  The diameter of
             the circle is given by the minimum of w,y.
-        function : Callable[[Light], [Color, Color]]
+        function : LightFunction
             The function returns the (ring, fill) colors
         """
         # The parent init sets convenience properties like self.x, etc
         super(Light, self).__init__(position, size)
 
+        if (function is not None
+                and not issubclass(function.__class__, LightFunction)):
+            raise ValueError(f"Function type {type(function)} is not a "
+                             f"LightFunction.")
         self.function = function
+
         self._radius = min(size[0], size[1]) >> 1
         self._center = (self._x + self._w // 2), (self._y + self._h // 2)
 
-    def render(self, surface):
+    def render(self, page, surface):
         if self.function is None:
             color1 = self.color1
             color2 = self.color2
         else:
-            color1, color2 = self.function(self)
+            color1, color2 = self.function.colors(page, self)
         # Fill
         pygame.draw.circle(surface, color2, self._center, self._radius, 0)
         # Outline ring
         pygame.draw.circle(surface, color1, self._center, self._radius, 2)
 
-    def can_focus(self):
-        return False
-
-
-class Indicator(IWidget):
-    """ Indicators are dynamic Labels, controlled by their function.
-    """
-
-    @doc_inherit(IWidget.__init__, style='numpy_with_merge')
-    def __init__(self, position, size,
-                 function=None,
-                 **kwargs):
-        """ Initialize the Indicator.
-
-        If color1 is None, the indicator is colored like a label, with only
-        the label colors being used.
-
-        If color1 is specified, then the indicator is set in a filled
-        rectangle with the active/inactive color.
-
-        Parameters
-        ----------
-        function : Callable[[Indicator], [bool, str]]
-            The function that defines the content and state of the indicator.
-            The bool is an 'active' flag, which sets the active color
-            palette if True.  The string sets the indicator label.  If it is
-            None, then the indicator defaults to self.label.
-        """
-        # Set up the boxing behavior and font size.
-        kw_font_size = kwargs.pop('font_size', None)
-        if kwargs.get('color1') is None:
-            self._box = False
-            font_size = kw_font_size or size[1]
-        else:
-            self._box = True
-            font_size = kw_font_size or (size[1] >> 1)
-        self.function = function
-
-        super(Indicator, self).__init__(position, size,
-                                        font_size=font_size,
-                                        **kwargs)
-
-    def render(self, surface):
-        if self.function:
-            active, label = self.function(self)
-        else:
-            active, label = False, None
-        # Default to the internal label
-        label = label or self.label
-
-        color, label_color = self.state_colors(active=active)
-
-        if self._box:
-            pygame.draw.rect(surface, color, (self.position, self.size), 0)
-
-        self.render_centered_text(surface, label, label_color)
-
-    def can_focus(self):
-        return False
-
 
 class Graph(IWidget):
     """ A dynamic graph.
@@ -192,34 +192,41 @@ class Graph(IWidget):
             specified, will calculate based on the visible data.
         quiet : Bool
             If set and True, quiets the graph (doesn't print the values)
-        function : Callable[[IWidget, int], List[dict]]
+        function : GraphFunction
             The update is called with the number of samples being requested.
             It should return `data` with that many samples or fewer.  If it
             returns fewer than 2 samples, that trace may not be rendered.
-            The dictionary in the list defines the color of the trace and
-            a list of float with the sample data:
-                {"color": <Color>, "data": [<float>]}
         """
         super(Graph, self).__init__(position, size, **kwargs)
+        if (function is not None
+                and not issubclass(function.__class__, GraphFunction)):
+            raise ValueError(f"Function type {type(function)} is not a "
+                             f"GraphFunction.")
+        self.function = function
+
         self.min_max = min_max
         self.quiet = quiet
-        self.function = function
-        if self.function is not None:
-            # Determine how many samples the update method returns, so we can
-            # scale the font appropriately.
-            samples = self.function(self, 1)
-            ratio_h = max(2.5, len(samples))
-            self._graph_font = pygame.font.Font(FONT_NAME,
-                                                int(self._h / ratio_h + 0.5))
+        self._initialized = False
 
-    def render(self, surface):
+    def render(self, page, surface):
         """ Render the graph.
 
         The graph width determines how many data samples we ask for.
         The graph(s) are then drawn inside of the rectangle, with the last
-        values in the graph optinally data rendered as labels to the right
+        values in the graph optionally data rendered as labels to the right
         (but inside) of the graph.
         """
+        if not self._initialized:
+            if self.function is not None:
+                # Determine how many samples the update method returns, so we
+                # can scale the font appropriately.
+                samples = self.function.samples(page, self, 1)
+                ratio_h = max(2.5, len(samples))
+                self._graph_font = pygame.font.Font(
+                    FONT_NAME,
+                    int(self._h / ratio_h + 0.5))
+            self._initialized = True
+
         graph_pos = (self._x, self._y)
         graph_size = (self._w, self._h)
         pygame.draw.rect(surface, self.color1, (graph_pos, graph_size), 2)
@@ -228,7 +235,7 @@ class Graph(IWidget):
         if self.function is None:
             return
 
-        values = self.function(self, graph_size[0] - 2)
+        values = self.function.samples(page, self, graph_size[0] - 2)
         if self.min_max is None:
             min_val = 9999
             max_val = -9999
@@ -245,7 +252,6 @@ class Graph(IWidget):
             min_val = self.min_max[0]
             max_val = self.min_max[1]
 
-
         range = max_val - min_val
         # Range must be > 0 so adjust around the existing min/max
         if range < .01:
@@ -295,6 +301,3 @@ class Graph(IWidget):
                     else:
                         textpos.centery = prev_y
                     surface.blit(text, textpos)
-
-        if self._focus:
-            self.render_focus(surface)
